@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2011-2015 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2011-2017 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -46,6 +46,10 @@
   typedef char *range_id_t;
 #endif
 
+#if defined(MRN_MARIADB_P) || MYSQL_VERSION_ID < 80002
+  typedef st_select_lex SELECT_LEX;
+#endif
+
 #if MYSQL_VERSION_ID >= 50609
 #  define MRN_KEY_HAS_USER_DEFINED_KEYPARTS
 #endif
@@ -356,4 +360,60 @@
 #  define MRN_HAVE_MYSQL_FIELD_PART_OF_KEY_NOT_CLUSTERED
 #endif
 
+#if defined(MRN_MARIADB_P) &&                                           \
+  ((MYSQL_VERSION_ID >= 100207) ||                                      \
+   ((MYSQL_VERSION_ID >= 50557) && (MYSQL_VERSION_ID < 100000)))
+#  define mrn_create_partition_name(out,                                \
+                                    out_length,                         \
+                                    in1,                                \
+                                    in2,                                \
+                                    name_variant,                       \
+                                    translate)                          \
+  create_partition_name(out, out_length, in1, in2, name_variant, translate)
+#  define mrn_create_subpartition_name(out,             \
+                                       out_length,      \
+                                       in1,             \
+                                       in2,             \
+                                       in3,             \
+                                       name_variant)    \
+  create_subpartition_name(out, out_length, in1, in2, in3, name_variant)
+#else
+#  define mrn_create_partition_name(out,                                \
+                                    out_length,                         \
+                                    in1,                                \
+                                    in2,                                \
+                                    name_variant,                       \
+                                    translate)                          \
+  (create_partition_name(out, in1, in2, name_variant, translate), 0)
+#  define mrn_create_subpartition_name(out,             \
+                                       out_length,      \
+                                       in1,             \
+                                       in2,             \
+                                       in3,             \
+                                       name_variant)    \
+  (create_subpartition_name(out, in1, in2, in3, name_variant), 0)
+#endif
+
+#if (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80002)
+#  define ITEM_SUM_GET_NEST_LEVEL(sum_item) (sum_item)->base_select->nest_level
+#  define ITEM_SUM_GET_AGGR_LEVEL(sum_item) (sum_item)->aggr_select->nest_level
+#  define ITEM_SUM_GET_MAX_AGGR_LEVEL(sum_item) (sum_item)->max_aggr_level
+#else
+#  define ITEM_SUM_GET_NEST_LEVEL(sum_item) (sum_item)->nest_level
+#  define ITEM_SUM_GET_AGGR_LEVEL(sum_item) (sum_item)->aggr_level
+#  define ITEM_SUM_GET_MAX_AGGR_LEVEL(sum_item) (sum_item)->max_arg_level
+#endif
+
+#if (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80002)
+  typedef bool mrn_bool;
+#else
+  typedef my_bool mrn_bool;
+#endif
+
+#define MRN_ALLOCATE_VARIABLE_LENGTH_ARRAYS(type, variable_name, variable_size) \
+  type *variable_name =                                                 \
+    (type *)mrn_my_malloc(sizeof(type) * (variable_size), MYF(MY_WME))
+#define MRN_FREE_VARIABLE_LENGTH_ARRAYS(variable_name) \
+  my_free(variable_name)
+
 #endif /* MRN_MYSQL_COMPAT_H_ */
